importPackage(java.lang);
importClass(Packages.com.alarmpoint.integrationagent.apxml.APXMLMessage);
importClass(Packages.com.alarmpoint.integrationagent.apxml.APXMLToken);

/**
 * This is the main method for handling APXML messages sent to the
 * Integration Service by APClient.  The messages may be requests to perform
 * local activity, or they may be requests to make submissions to AlarmPoint.
 * <p>
 * Any APXMLMessage object that this method returns will be sent to AlarmPoint
 * via the Integration Service's outbound queues.
 */
function apia_input(apxml)
{
  var eventType = apxml.hasToken("event_type") ? apxml.getValue("event_type") : "add"
  
  if ("del".equalsIgnoreCase(eventType))
  {
    return processDelEvent(apxml);
  }
  else
  {
    return processAddEvent(apxml);
  } 
}

/**
 * Enhances the APXML for the event injection
 * @param apxml APXML Message that was received by the script
 * @return enhanced APXML Message
 */
function processAddEvent(apxml)
{
  // injected message fields
  var msgSrc = apxml.getValue("msg_source");

  if (msgSrc.equals("unknown0"))
  {
    apxml.setToken("msg_source", "");  // set to empty as external nodes do not have polices
    ServiceAPI.getLogger().info("Enrichment Complete for External Node");
  } else
  {
    // strip off the version
    if (msgSrc.lastIndexOf("(") > 0)
    {
      msgSrc = msgSrc.substring(0, msgSrc.lastIndexOf("("));
    }
    apxml.setToken("msg_source", msgSrc);
    ServiceAPI.getLogger().info("Enrichment Complete for Managed Node");
  }
  return apxml;
}

/**
 * Create a new APXML Message to sent to the current event domain for the DEL
 * @param apxml APXML Message that was received by the script
 * @return APXML Message for the delete event
 */
function processDelEvent(apxml)
{
  var incidentId = apxml.getValue("incident_id");
  var request = ServiceAPI.createAPXML();
  request.setMethod("del");
  request.setSubclass("action");
  request.setToken(APXMLMessage.AGENT_CLIENT_ID, apxml.getValue(APXMLMessage.AGENT_CLIENT_ID));
  request.setToken(APXMLMessage.APIA_PROCESS_GROUP, incidentId);
  request.setToken("incident_id", incidentId);

  return request;
} 

/**
 * This is the main method for handling APXML messages sent to the
 * Integration Service by AlarmPoint.  The messages may be responses to
 * previous submissions, or they may be requests to the Integration Service
 * that originate from AlarmPoint
 * (e.g., via an ExternalServiceMessage/ExternalServiceRequest2).
 * <p>
 * Any APXMLMessage object that this method returns will be sent to AlarmPoint
 * either via the Integration Service's outbound queues or directly as a
 * Web Service response, depending on the mechanism used to submit the APXML.
 */
function apia_response(apxml)
{
  var method = apxml.getMethod();
  if (APXMLMessage.OK_METHOD.equalsIgnoreCase(method))
  {
    return handleOK(apxml);
  }
  else if (APXMLMessage.ERROR_METHOD.equalsIgnoreCase(method))
  {
    return handleERROR(apxml);
  }
  else if (APXMLMessage.SEND_METHOD.equalsIgnoreCase(method))
  {
    return handleSend(apxml);
  }
  else if (APXMLMessage.REQUEST_METHOD.equalsIgnoreCase(method))
  {
    return handleRequest(apxml);
  }
  else
  {
    throw new IllegalArgumentException("Unrecognized APXML method: " + method);
  }
}


/**
 * OK responses are returned by AlarmPoint for each APXML message that it accepts
 * from an Integration Service.
 * <p>
 * This implementation just logs the ack.
 */
function handleOK(apxml)
{
  var id = apxml.getTransactionId();
  ServiceAPI.getLogger().info("Submission of transaction " + id + "succeeded.");
}


/**
 * ERROR responses are returned by AlarmPoint for each APXML message that it
 * is unable to accept from an Integration Service.
 * <p>
 * This implementation just logs the error.
 */
function handleERROR(apxml)
{
  var transaction = apxml.getTransactionId();
  var code = apxml.getValue(APXMLMessage.ERROR_CODE);
  ServiceAPI.getLogger().error("Submission of transaction " + transaction + "failed with error code " + code);
}


/**
 * This implementation just logs que request, there shouldn't be any direct calls to this method
 */
function handleRequest(apxml)
{
  ServiceAPI.getLogger().error("Service received unexpected 'Request' with message: " + apxml);
}


var successRequest = "HP Operations Manager (Win) Request Completed";

/**
 * The main routine for handling Requests starts here.
 * <p>
 * Throw an exception that will be logged by the Integration Agent.
 */
function handleSend(apxml)
{
  // message fields
  var incidentId = apxml.getValue("incident_id");
  var requestText = apxml.getValue("request_text");
  var message = apxml.getValue("message");
  var requestOwner = apxml.getValue("request_owner");
  var omwOwner = apxml.getValue("omw_owner");
  var omwPassword = apxml.getValue("omw_password");
  var requestValue = apxml.getValue("request_value");
  var messageText = apxml.getValue("message_text");

  ServiceAPI.getLogger().debug("incident_id: " + incidentId + " Request Text: " + requestText);
  if (incidentId)
  {
    // Convert Callout RAS Injection Messages to Annotation Requests
    if (message)
    {
      apxml.setToken("request_text", "ApOmwAnnotate");
      apxml.setToken("message_text", message);
    }

    if (messageText)
    {
      apxml.setToken("message_text", "[ AlarmPoint ] " + messageText);
    } else
    {
      apxml.setToken("message_text", "");
    }

    if (requestText)
    {
      var commandList = [
          "cmd",
          "/c",
          "cscript.exe",
          "/Nologo",
          "integrationservices\\hpomw\\OmwApt.vbs",
          "-MsgId", incidentId,
          "-Request"
      ];
      if (requestText.equalsIgnoreCase("ApOmwAck"))
      {
        commandList = commandList.concat([
              requestText,
              "-arg1", "\"" + messageText + "\"",
              "-omwuser", "\"" + omwOwner + "\"",
              "-omwpassword", "\"" + omwPassword + "\""
          ]);
          execute(commandList, incidentId, requestText);
      }
      else if (requestText.equalsIgnoreCase("ApOmwOwn"))
      {
        if (requestOwner)
        {
          commandList = commandList.concat([
              requestText,
              "-arg1", requestOwner,
              "-arg2", "\"" + messageText + "\"",
              "-omwuser", "\"" + omwOwner + "\"",
              "-omwpassword", "\"" + omwPassword + "\""
          ]);
          execute(commandList, incidentId, requestText);
        }
        else
        {
          ServiceAPI.getLogger().error("Agent Client: hp_operations_manager_win - Request Script Failed for (" + requestText + "/" + incidentId + "): No User Specified to Own the Incident.");
        }
      }
      else if (requestText.equalsIgnoreCase("ApOmwDisown"))
      {
        if (requestOwner)
        {
          commandList = commandList.concat([
              requestText,
              "-arg1", requestOwner,
              "-arg2", "\"" + messageText + "\"",
              "-omwuser", "\"" + omwOwner + "\"",
              "-omwpassword", "\"" + omwPassword + "\""
          ]);
          execute(commandList, incidentId, requestText);
        }
        else
        {
          ServiceAPI.getLogger().error("Agent Client: hp_operations_manager_win - Request Script Failed for (" + requestText + "/" + incidentId + "): No User Specified to Own the Incident.");
        }
      }
      else if (requestText.equalsIgnoreCase("ApOmwChgSev"))
      {
        ServiceAPI.getLogger().info("Entering OMW Change Sev");
        ServiceAPI.getLogger().debug("-omwuser = " + omwOwner);

        if (requestValue)
        {
          commandList = commandList.concat([
              requestText,
              "-arg1", requestValue,
              "-arg2", "\"" + messageText + "\"",
              "-omwuser", "\"" + omwOwner + "\"",
              "-omwpassword", "\"" + omwPassword + "\""
          ]);
          execute(commandList, incidentId, requestText);
        }
        else
        {
          ServiceAPI.getLogger().error("Agent Client: hp_operations_manager_win - Request Script Failed for (" + requestText + "/" + incidentId + "): Missing Severity Value for Request.");
        }
      }
      else if (requestText.equalsIgnoreCase("ApOmwAnnotate"))
      {
        if (messageText != "")
        {
          ServiceAPI.getLogger().info(messageText);
          commandList = commandList.concat([
              requestText,
              "-arg1", "\"" + messageText + "\"",
              "-omwuser", "\"" + omwOwner + "\"",
              "-omwpassword", "\"" + omwPassword + "\""
          ]);
          execute(commandList, incidentId, requestText);
        }
        else
        {
          ServiceAPI.getLogger().error("Agent Client: hp_operations_manager_win - Request Script Failed for (" + requestText + "/" + incidentId + "): Missing Annotation Text.");
        }
      }
      else
      {
        ServiceAPI.getLogger().error("Agent Client: hp_operations_manager_win - Request Script Failed for (" + incidentId + "): Unknown processing request: " + requestText);
      }
    }
    else
    {
      ServiceAPI.getLogger().info("Agent Client: hp_operations_manager_win - Request Script Failed for (" + incidentId + "): Required request missing from trigger message: " + APXML_MESSAGE);
    }
  }
  else
  {
    ServiceAPI.getLogger().error("Agent Client: hp_operations_manager_win - Request Script Failed: Request not associated with an Incident.");
  }
}


/**
 * This procedure will run an external command, providing it the required
 * data tags.
 *
 * GIVEN INPUTS
 * String command - The full path and name of a program to execute, plus
 * any command line arguments.
 *
 * incidentId - The unique ID associated with the event.
 *
 * requestText - The choice selected by the person in response to the event.
 */
function execute(command, incidentId, requestText)
{
  // Execute an external command, and wait for it to finish.  By default
  // we append the incident ID so the management system knows what
  // event AlarmPoint is referring to.
  var p = Runtime.getRuntime().exec(command);
  p.waitFor();
  ServiceAPI.getLogger().info("Agent Client: hp_operations_manager_win - Request Script Executed for (" + requestText + "/" + incidentId + "), exit value: " + p.exitValue());
}
